/**
 * Netwosix Web Admin - agent-info
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */

#include <agent.h>

#include <unistd.h>
#include <sys/utsname.h>



/**
 * This function is called automatically for every request recevide on stdin.
 * The XML data returned is automatically inserted in the response and the
 * written on stdout. 
 */
AlString * agent_test_response_create(const AgentRequest * request, const char * data)
{
	AlXmlList      * list = al_xml_list_new();
	AlXmlNode      * node;
	size_t           hostname_len = 100;
	char             hostname[hostname_len];
	struct utsname   buf;



	uname(&buf);
	gethostname(hostname, hostname_len);

	/*
	 * <info>
	 *	<uname sysname="" nodename="" release="" version="" machine=""/>
	 *	<network hostname=""/>
	 * </info>
	 */

	list->node = al_xml_node_new();
	al_xml_node_set_name(list->node, "info");

	node = al_xml_node_new();

	al_xml_node_set_name(node, "uname");
	al_xml_node_attribute_new(node, "sysname", buf.sysname);
	al_xml_node_attribute_new(node, "nodename", buf.nodename);
	al_xml_node_attribute_new(node, "release", buf.release);
	al_xml_node_attribute_new(node, "version", buf.version);
	al_xml_node_attribute_new(node, "machine", buf.machine);

	al_xml_list_node_add(list, node);

	node = al_xml_node_new();

	al_xml_node_set_name(node, "network");
	al_xml_node_attribute_new(node, "hostname", hostname);

	al_xml_list_node_add(list, node);

	return (list) ? al_xml_list_to_string(list, NULL, NULL) : NULL;
}


AGENT_DEFAULT_MAIN("agent-info", agent_test_response_create);
