/**
 * Netwosix Web Admin - agentd
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */


#ifndef REQUEST_H_
#define REQUEST_H_


/**
 * Structure with a request
 */
struct _AgentdRequest
{

	struct _AgentdRequest * prev;
	struct _AgentdRequest * next;

	char                  * id;          /* request id              */
	char                  * agent_name;  /* name of requested agent */
	char                  * agent_data;  /* data for agent          */

};

typedef struct _AgentdRequest AgentdRequest;


/**
 */
#define agentd_request_set_id(request,v)                  (request)->id = al_strdup(v)


/**
 */
#define agentd_request_get_id(request)                    (request)->id


/**
 */
#define agentd_request_set_agent_name(request,v)          (request)->agent_name = al_strdup(v)


/**
 */
#define agentd_request_get_agent_name(request)            (request)->agent_name


/**
 */
#define agentd_request_set_agent_data(request,v)          (request)->agent_data = al_strdup(v)


/**
 */
#define agentd_request_get_agent_data(request)            (request)->agent_data


/**
 * Structure with a list of request.
 */
struct _AgentdRequestPacket
{

	struct _AgentdRequest * requests; /* list of request in the packet */

};

typedef struct _AgentdRequestPacket AgentdRequestPacket;


/**
 * Create a new request.
 *
 * @return the new request.
 */
AgentdRequest * agentd_request_new(void);


/**
 * Delete a request.
 *
 * @param request request to free
 */
void agentd_request_delete(AgentdRequest * request);


/**
 * Create a new request packet.
 *
 * @return the new request packet.
 */
AgentdRequestPacket * agentd_request_packet_new(void);


/**
 * Delete a request packet and all request in list.
 *
 * @param request_packet request packet to free
 */
void agentd_request_packet_delete(AgentdRequestPacket * request_packet);


/**
 * Add a request to a request packet.
 *
 * @param request_packet request packet
 * @param request request to add
 */
void agentd_request_packet_request_add(AgentdRequestPacket * request_packet, AgentdRequest * request);


/**
 * Create a new request packet from xml string.
 *
 * @param xml xml string
 *
 * @return a new request packet or NULL if xml is not valid.
 */
AgentdRequestPacket * agentd_request_packet_from_xml_string(const AlString * xml);


/**
 * Create a new request packet from xml.
 *
 * @param xml xml
 *
 * @return a new request packet or NULL if xml is not valid.
 */
AgentdRequestPacket * agentd_request_packet_from_xml(AlXml * xml);


/**
 * Create a new request from xml.
 *
 * @param node xml node
 *
 * @return a new request
 */
AgentdRequest * agentd_request_from_xml(AlXmlNode * node);



#endif /* REQUEST_H_ */
