/**
 * Netwosix Web Admin - libagent
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */


#ifndef RESPONSE_H_
#define RESPONSE_H_


/**
 * Type of response
 */
enum _AgentResponseType
{

	AGENT_RESPONSE_OK                 = 0, /* everything is ok                */
	AGENT_RESPONSE_AGENT_FAILED       = 1, /* agent end in error              */
	AGENT_RESPONSE_AGENT_INVALID_DATA = 2  /* agent end in error              */

};

typedef int AgentResponseType;


/**
 * Structure representing a response
 */
struct _AgentResponse
{

	struct _AgentResponse * prev;
	struct _AgentResponse * next;

	char                   * id;         /* request id              */
	AgentResponseType        type;       /* type of response        */
	char                   * agent_data; /* data for agent          */

};

typedef struct _AgentResponse AgentResponse;


/**
 */
#define agent_response_set_id(response,v)                   (response)->id = al_strdup(v)


/**
 */
#define agent_response_get_id(response)                    (response)->id


/**
 */
#define agent_response_set_agent_data(response,v)          (response)->agent_data = al_strdup(v)


/**
 */
#define agent_response_get_agent_data(response)            (response)->agent_data


/**
 */
#define agent_response_set_type(response,v)                (response)->type = v


/**
 */
#define agent_response_get_type(response)                  (response)->type


/**
 * Structure representing a packet with one or many response.
 */
struct _AgentResponsePacket
{

	struct _AgentResponse * responses; /* list of response in the packet */

};

typedef struct _AgentResponsePacket AgentResponsePacket;


/**
 * Create a new response.
 *
 * @return the new response.
 */
AgentResponse * agent_response_new(void);


/**
 * Delete a response.
 *
 * @param response response to free
 */
void agent_response_delete(AgentResponse * response);


/**
 * Create a new response packet.
 *
 * @return the new response packet.
 */
AgentResponsePacket * agent_response_packet_new(void);


/**
 * Delete a response packet and all response in list.
 *
 * @param response_packet response packet to free
 */
void agent_response_packet_delete(AgentResponsePacket * response_packet);


/**
 * Add a response to a response packet.
 *
 * @param response_packet response packet
 * @param response response to add
 */
void agent_response_packet_response_add(AgentResponsePacket * response_packet, AgentResponse * response);


/**
 */
typedef AlString * (*agent_response_create_func)(const AgentRequest * request, const char * data);


/**
 * Create a new response basend on a request.
 * Before to create the response call the func. The string
 * returned by func is passed as "data".
 *
 * @param request request who response is based
 * @param func func to call for every request
 *
 * @return the new response
 */
AgentResponse * agent_response_from_request(const AgentRequest * request, agent_response_create_func func);


/**
 * Create a new response packet basend on a request packet.
 * This process every request in the packet.
 *
 * @param request_packet request packet who response is based
 * @param func func to call for every request
 *
 * @return the new response packet
 */
AgentResponsePacket * agent_response_packet_from_request_packet(const AgentRequestPacket * request_packet, agent_response_create_func func);


/**
 * Dump the response packet to xml.
 *
 * @param response_packet response packet to dump
 *
 * @return response packet in xml
 */
AlXmlList * agent_response_packet_to_xml(const AgentResponsePacket * response_packet);


/**
 * Dump the response packet to xml.
 *
 * @param response_packet response packet to dump
 *
 * @return response packet in xml
 */
AlString * agent_response_packet_to_xml_string(const AgentResponsePacket * response_packet);


/**
 * Dump the response to xml.
 *
 * @param response response to dump
 *
 * @return response in xml node
 */
AlXmlNode * agent_response_to_xml(const AgentResponse * response);


#endif /* RESPONSE_H_ */
