/**
 * Netwosix Web Admin - libal
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */

#include "al.h"

#include <spawn.h>
#include <sys/types.h>
#include <sys/stat.h>


#define AL_SPAWN_SU_CMD           "su -c %s - %s"


/**
 */
int al_spawn(const char * bin, const char * user, const char * std_in, AlString * std_out, int * status)
{
	char                       * r_bin = NULL;
	char                         buf;
	int                          sts;
	int                          i;
	int                          len;
	int                          err;
	pid_t                        pid;
	posix_spawn_file_actions_t   actions;
	posix_spawnattr_t            attrs;
	int                          ifd[2];
	int                          ofd[2];


	al_return_val_if_fail(bin, 99);


	if ((err = posix_spawn_file_actions_init (&actions)) != 0)
	{
		fprintf(stderr, "Error %d executing posix_spawn_file_actions_init\n", err);
		return 1;
	}


	/* Create pipe for redirect stdin e stdout */
	if (pipe (ifd) == -1)
	{
		fprintf(stderr, "Error %d executing pipe\n", errno);
		return 2;
	}

	if (pipe (ofd) == -1)
	{
		fprintf(stderr, "Error %d executing pipe\n", errno);
		return 3;
	}

	posix_spawn_file_actions_adddup2 (&actions, ofd[0], STDIN_FILENO);
	posix_spawn_file_actions_adddup2 (&actions,ifd[1], STDOUT_FILENO);
	posix_spawn_file_actions_addclose (&actions, ofd[0]);
	posix_spawn_file_actions_addclose (&actions, ifd[1]);
	posix_spawn_file_actions_addclose (&actions, ofd[1]);
	posix_spawn_file_actions_addclose (&actions, ifd[0]);


	if (std_in)
	{
		write(ofd[1], std_in, strlen(std_in));
		write(ofd[1], "\n", 1);
	}

	if (user)
	{
		r_bin = (char*)malloc(sizeof(char) * ( strlen(user) + strlen(bin) + strlen(AL_SPAWN_SU_CMD) +1));
		sprintf(r_bin, AL_SPAWN_SU_CMD, bin, user);
		if ((err = posix_spawn (&pid, bin, &actions, NULL, NULL, NULL)) != 0)
		{
			fprintf(stderr, "Error %d during posix_spawn\n", err);
			al_delete(r_bin);
			return 4;
		}
		al_delete(r_bin);
	}
	else
	{
		if ((err = posix_spawn (&pid, bin, &actions, NULL, NULL, NULL)) != 0)
		{
			fprintf(stderr, "Error %d during posix_spawn\n", err);
			return 4;
		}
	}

	close(ifd[1]);
	close(ofd[0]);
	close(ofd[1]);

	if (std_out == NULL)
		close(ifd[0]);
	
	/* .. and wait to end */
	waitpid(pid, &sts, 0);

	if (status) (*status) = sts;

	if (std_out)
	{
		while (read(ifd[0], &buf, 1) > 0)
			al_string_append_char(std_out, buf);

		close(ifd[0]);
	}
	

	return 0;
}
