/**
 * Netwosix Web Admin - libal
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */

#ifndef CONNECTION_H_
#define CONNECTION_H_


typedef struct _AlConnectionManager AlConnectionManager;
typedef struct _AlConnection        AlConnection;


/**
 */
typedef void(*al_connection_manager_delete_func)(AlConnectionManager *);


/**
 */
typedef AlConnection*(*al_connection_wait_func)(AlConnectionManager *);


/**
 */
typedef AlConnection*(*al_connection_connect_func)(AlConnectionManager *, const char *);


/**
 */
typedef void(*al_connection_close_func)(AlConnection *);


/**
 */
typedef AlString*(*al_read_func)(AlConnection *);


/**
 */
typedef void(*al_send_func)(AlConnection *, const AlString *);


/**
 */
struct _AlConnectionManager
{

	al_connection_wait_func             wait;
	al_read_func                        read;
	al_send_func                        send;
	al_connection_close_func            close;
	al_connection_manager_delete_func   del;
	al_connection_connect_func          conn;

	int                                 fork;
	void                              * data;

};


/**
 */
struct _AlConnection
{

	AlConnectionManager * connection_manager; 

	void                * data;
};


/**
 * Wait for a new connection.
 *
 * @param connection_manager the connection manager
 *
 * @return return the new connection or NULL if there is no more new connection.
 */
#define al_connection_wait(connection_manager)                \
 (connection_manager->wait(connection_manager))


/**
 * Close an open connection.
 *
 * @param connection connection to close
 */
#define al_connection_close(connection)                        \
  (connection->connection_manager->close(connection))


/**
 * Return true if the connection manager need a fork when
 * a new connection is estabilished.
 *
 * @param connection_manager the connection manager
 *
 * @return true if a fork is needed
 */
#define al_connection_need_fork(connection_manager)            \
 (connection_manager->fork)


/**
 * Read a packet from a connection.
 *
 * @param connection connection to use for reading
 *
 * @return the new packet or NULL if there is no more data
 */
#define al_connection_read(connection)                         \
 (connection->connection_manager->read(connection))


/**
 * Send a packet at the given connection.
 *
 * @param connection connection to use for sendig
 * @param packet packet to send
 */
#define al_connection_send(connection, packet)                 \
 (connection->connection_manager->send(connection, packet))


/**
 * Delete a connection manager and close all open conenction.
 *
 * @param connection_manager the connection manager
 */
#define al_connection_manager_delete(connection_manager)       \
 (connection_manager->del(connection_manager))


/**
 * Connect to a server.
 *
 * @param connection_manager the connection manager
 * @param server server to connect
 *
 * @return the new connection or NULL if fail
 */
#define al_connection_connect(connection_manager,server)       \
 (connection_manager->conn(connection_manager,server))


/**
 * Create a connection object from it's name
 *
 * @param connection_type type of connection. If NULL is 
 * take from the attribute "connection" in node
 * @param supported NULL terminated list of supported connection
 * @param node node with connection properties
 *
 * @return the connection manager or NULL if vail
 */
AlConnectionManager * al_connection_manager_from_string(const char * connection_type, const char * supported[], AlXmlNode * node);


#endif /* CONNECTION_H_ */
